<?php

/**
 * @file
 * Implements the revision reference relationship for Panels.
 */

/**
 * Implementation of hook_ctools_relationships().
 */
function revisionreference_node_from_revisionref_ctools_relationships() {
  return array(
    'title' => t('Node from revisionreference'),
    'keyword' => 'revisionreference',
    'description' => t('Adds a node from a revisionreference in a node context; if multiple revisions are referenced, this will get the node of the first referenced revision only.'),
    'required context' => new ctools_context_required(t('Node'), 'node'),
    'context' => 'revisionreference_node_from_revisionref_context',
    'settings form' => 'revisionreference_node_from_revisionref_settings_form',
    'settings form validate' => 'revisionreference_node_from_revisionref_settings_form_validate',
  );
}

/**
 * Return a new ctools context based on an existing context.
 */
function revisionreference_node_from_revisionref_context($context, $conf) {
  $field = content_fields($conf['field_name']);

  // If unset it wants a generic, unfilled context, which is just NULL.
  if (empty($context->data)) {
    $new_context = ctools_context_create_empty('node', NULL);
  }
  else if (isset($context->data->{$conf['field_name']}[0]['vid']) && ($vid = $context->data->{$conf['field_name']}[0]['vid'])) {
    if ($node = node_load(array('status' => 1), $vid)) {
      $new_context = ctools_context_create('node', $node);
    }
  }

  if (!empty($new_context)) {
    // Have revisionreference relationships limit CCK field availability as well.
    $restrictions = array_keys(array_filter($field['referenceable_revision_types']));
    if ($restrictions) {
      if (isset($new_context->restrictions['type'])) {
        $new_context->restrictions['type'] = array_unique(array_merge($new_context->restrictions['type'], $restrictions));
      }
      else {
        $new_context->restrictions['type'] = $restrictions;
      }
    }

    return $new_context;
  }
}

/**
 * Settings form for the ctools relationship.
 */
function revisionreference_node_from_revisionref_settings_form($conf) {
  $options = array();
  foreach (content_fields() as $field) {
    if ($field['type'] == 'revisionreference') {
      $options[$field['field_name']] = t($field['widget']['label']);
    }
  }
  $form['field_name'] = array(
    '#title' => t('Node Revision reference field'),
    '#type' => 'select',
    '#options' => $options,
    '#default_value' => isset($conf['field_name']) ? $conf['field_name'] : '',
    '#prefix' => '<div class="clear-block">',
    '#suffix' => '</div>',
  );

  return $form;
}
